#include <GL/freeglut.h>

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include <sixense.h>
#include <sixense_math.hpp>
#ifdef WIN32
#include <sixense_utils/mouse_pointer.hpp>
#endif
#include <sixense_utils/derivatives.hpp>
#include <sixense_utils/button_states.hpp>
#include <sixense_utils/event_triggers.hpp>
#include <sixense_utils/controller_manager/controller_manager.hpp>

#include <deque>


// flags that the controller manager system can set to tell the graphics system to draw the instructions
// for the player
static bool controller_manager_screen_visible = true;
std::string controller_manager_text_string;

int screenwidth = 1200, screenheight = 1200;

void glBasics()
{
	const GLfloat light_ambient[]  = { 0.0f, 0.0f, 0.0f, 1.0f };
	const GLfloat light_diffuse[]  = { 1.0f, 1.0f, 1.0f, 1.0f };
	const GLfloat light_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	const GLfloat light_position[] = { 2.0f, 5.0f, 5.0f, 0.0f };

	const GLfloat mat_ambient[]    = { 0.7f, 0.7f, 0.7f, 1.0f };
	const GLfloat mat_diffuse[]    = { 0.8f, 0.8f, 0.8f, 1.0f };
	const GLfloat mat_specular[]   = { 1.0f, 1.0f, 1.0f, 1.0f };
	const GLfloat high_shininess[] = { 100.0f };

	glClearColor(0.6f,0.6f,0.7f,1.0f);
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);

	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);

	glEnable(GL_LIGHT0);
	glEnable(GL_NORMALIZE);
	glEnable(GL_COLOR_MATERIAL);

	glLightfv(GL_LIGHT0, GL_AMBIENT,  light_ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE,  light_diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	glMaterialfv(GL_FRONT, GL_AMBIENT,   mat_ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE,   mat_diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR,  mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);
}

// Draw a text string, at the given row, column
static void shapesPrintf (int row, int col, const char *fmt, ...)
{
	static char buf[256];
	int viewport[4];
	void *font = GLUT_BITMAP_HELVETICA_12;
	va_list args;

	va_start(args, fmt);
#if defined(WIN32) && !defined(__CYGWIN__)
	(void) _vsnprintf (buf, sizeof(buf), fmt, args);
#else
	(void) vsnprintf (buf, sizeof(buf), fmt, args);
#endif
	va_end(args);

	glGetIntegerv(GL_VIEWPORT,viewport);

	glPushMatrix();
	glLoadIdentity();

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	glOrtho(0,viewport[2],0,viewport[3],-1,1);

	glRasterPos2i
		(
		glutBitmapWidth(font, ' ') * col,
		- glutBitmapHeight(font) * (row+0) + viewport[3]
	);
	glutBitmapString (font, (unsigned char*)buf);

	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}


// Handle resizing of the window
static void	resize(int width, int height)
{
	screenwidth = width;
	screenheight = height;

}



// Draw the two 3d objects representing the controllers
static void drawObjects() {

	int base, cont, i, j;
	sixenseAllControllerData acd;
	float rot_mat[4][4];

	glViewport(0, 0, screenwidth, screenheight);

	sixenseSetActiveBase(0);
	// Get the latest controller data
	sixenseGetAllNewestData( &acd );
	double eyex, eyey, eyez;
	eyex = 	acd.controllers[0].pos[0]/1.0f;
	eyey = 	acd.controllers[0].pos[1]/1.0f;
	eyez = 	acd.controllers[0].pos[2]/1.0f;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// work in pixels
/*
	double pxpermm = 1280 / 288.0;
	// screen origin is at (0,-30, -360)
	glFrustum(((-screenwidth/2.0) / pxpermm - eyex)/2.0, ((screenwidth/2.0) / pxpermm - eyex)/2.0, 
		((-screenheight/2.0) / pxpermm - (eyey - 30.0))/2.0, ((screenheight/2.0) / pxpermm - (eyey-30.0))/2.0, 
		(360.0 + eyez)/2.0, 360.0 + eyez + 1200.0);
*/
		// screen origin is at (0,220, -360)
	double pxpermm = 1600 / 400.0;
	glFrustum(((-screenwidth/2.0) / pxpermm - eyex)/2.0, ((screenwidth/2.0) / pxpermm - eyex)/2.0, 
		((-screenheight/2.0) / pxpermm - (eyey - 220.0))/2.0, ((screenheight/2.0) / pxpermm - (eyey-220.0))/2.0, 
		(160.0 + eyez)/2.0, 160.0 + eyez + 2200.0);


	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity() ;
	gluLookAt( eyex, eyey-220.0, 160.0 + eyez, eyex, eyey-220.0, 0.0, 0.0, 1.0, 0.0);

	glClearColor(0.6f,0.6f,0.7f,1.0f);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glEnable(GL_LIGHTING);

	glColor3d(0.2,0.2,0.2);

	glPushMatrix();
	
	glPushMatrix();
	glScalef( 1.0, 1.0, 5.0);
	glTranslatef(0.0, 0.0, (-screenwidth / pxpermm)/2.0);
	glutWireCube( screenwidth / pxpermm );
	glPopMatrix();

	glTranslatef(0.0, 0.0, -50.0);
	// Draw a sphere at the origin
	glutSolidSphere( 25.0, 18, 18 );

	glPushMatrix();
	glTranslatef( 20.0, 20.0, -200.0);
	glColor3d(0.8,0.1,0.1);
	glutSolidSphere( 25.0, 18, 18 );
	glTranslatef( 0.0, 0.0, -500.0);
	glColor3d(0.8,0.8,0.8);
	glutSolidCylinder( 5, 500, 15, 15 );
	glPopMatrix();

	glPushMatrix();
	glTranslatef( -30.0, 10.0, 100.0);
	glColor3d(0.3,0.8,0.1);
	glutSolidSphere( 20.0, 18, 18 );
	glTranslatef( 0.0, 0.0, -800.0);
	glColor3d(0.8,0.8,0.8);
	glutSolidCylinder( 5, 800, 15, 15 );
	glPopMatrix();


	glPopMatrix();


/*	
				
					glColor3d(1.0,1.0,1.0);

					glPushMatrix();
					for( i=0; i<3; i++ ) 
						for( j=0; j<3; j++ ) 
							rot_mat[i][j] = acd.controllers[cont].rot_mat[i][j];

					rot_mat[0][3] = 0.0f;
					rot_mat[1][3] = 0.0f;
					rot_mat[2][3] = 0.0f;
					rot_mat[3][0] = acd.controllers[cont].pos[0]/1.0f;
					rot_mat[3][1] = acd.controllers[cont].pos[1]/1.0f;
					rot_mat[3][2] = acd.controllers[cont].pos[2]/1.0f;
					rot_mat[3][3] = 1.0f;
//					std::cout << "x:" << acd.controllers[cont].pos[0] << std::endl;
//					std::cout << "y:" << acd.controllers[cont].pos[1] << std::endl;
//					std::cout << "z:" << acd.controllers[cont].pos[2] << std::endl;
					glMultMatrixf( (GLfloat*)rot_mat );

					glutSolidSphere( 15, 15, 15 );
					glTranslated( 0, 0, -52 );
					glutSolidCylinder( 5, 52, 15, 15 );


					glPopMatrix();
					*/
			
}



// Write a bunch of instruction text, as well as the current position and rotation information
void draw_controller_info() {


	drawObjects();


	glDisable(GL_LIGHTING);
	glColor3d(0.1,0.1,0.4);

	int next_line = 1;

	// Update the text
	sixenseSetActiveBase(0);
	glColor3d(0.1,0.1,0.4);
	shapesPrintf (next_line++, 3, "Base 0: %d controllers", sixenseGetNumActiveControllers() );

	sixenseAllControllerData acd;

	sixenseGetAllNewestData( &acd );
	double eyex, eyey, eyez;
	eyex = 	acd.controllers[0].pos[0]/1.0f;
	eyey = 	acd.controllers[0].pos[1]/1.0f;
	eyez = 	acd.controllers[0].pos[2]/1.0f;
	shapesPrintf (next_line++, 3, "( %f, %f, %f)", eyex, eyey, eyez);



}

// This is the callback that gets registered with the sixenseUtils::controller_manager. It will get called each time the user completes
// one of the setup steps so that the game can update the instructions to the user. If the engine supports texture mapping, the 
// controller_manager can prove a pathname to a image file that contains the instructions in graphic form.
// The controller_manager serves the following functions:
//  1) Makes sure the appropriate number of controllers are connected to the system. The number of required controllers is designaged by the
//     game type (ie two player two controller game requires 4 controllers, one player one controller game requires one)
//  2) Makes the player designate which controllers are held in which hand.
//  3) Enables hemisphere tracking by calling the Sixense API call sixenseAutoEnableHemisphereTracking. After this is completed full 360 degree
//     tracking is possible.
void controller_manager_setup_callback( sixenseUtils::ControllerManager::setup_step step ) {

	if( sixenseUtils::getTheControllerManager()->isMenuVisible() ) {

		// Turn on the flag that tells the graphics system to draw the instruction screen instead of the controller information. The game
		// should be paused at this time.
		controller_manager_screen_visible = true;

		// Ask the controller manager what the next instruction string should be.
		controller_manager_text_string = sixenseUtils::getTheControllerManager()->getStepString();

		// We could also load the supplied controllermanager textures using the filename: sixenseUtils::getTheControllerManager()->getTextureFileName();

	} else {

		// We're done with the setup, so hide the instruction screen.
		controller_manager_screen_visible = false;

	}

}

// Draw the grey screen with a single yellow line of text to prompt the user through the setup steps.
void draw_controller_manager_screen() {

	glClearColor( 0.2f, 0.2f, 0.2f, 1.0f );
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glColor3d(0.8,0.8,0.0);
	shapesPrintf(5,5, controller_manager_text_string.c_str() );

}


// glut calls this function each frame
static void display(void)
{

	// update the controller manager with the latest controller data here
	sixenseSetActiveBase(0);
	sixenseAllControllerData acd;
	sixenseGetAllNewestData( &acd );
	sixenseUtils::getTheControllerManager()->update( &acd );

	// Either draw the controller manager instruction screen, or display the controller information
	if( controller_manager_screen_visible ) {
		draw_controller_manager_screen();
	} else {
		draw_controller_info();
	}

	glutSwapBuffers();

}

static void
	key(unsigned char key, int x, int y)
{
	switch (key)
	{
	case 27 :
	case 'Q':
	case 'q': glutLeaveMainLoop () ;      break;


	default:
		break;
	}
}

static void
	idle(void)
{
	glutPostRedisplay();
}


int main(int argc, char *argv[])
{

	glutInitWindowSize(screenwidth, screenheight);
	glutInitWindowPosition(40,40);
	glutInit(&argc, argv);
	//glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH | GLUT_MULTISAMPLE);
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);

	glutCreateWindow("Sixense Test");

	glutReshapeFunc(resize);
	glutDisplayFunc(display);
	glutKeyboardFunc(key);
	glutIdleFunc(idle);

	glutSetOption ( GLUT_ACTION_ON_WINDOW_CLOSE, GLUT_ACTION_CONTINUE_EXECUTION ) ;

	glBasics();

	// Init sixense
	sixenseInit();

	// Init the controller manager. This makes sure the controllers are present, assigned to left and right hands, and that
	// the hemisphere calibration is complete.
	sixenseUtils::getTheControllerManager()->setGameType( sixenseUtils::ControllerManager::ONE_PLAYER_TWO_CONTROLLER );
	sixenseUtils::getTheControllerManager()->registerSetupCallback( controller_manager_setup_callback );

	glutMainLoop();

	sixenseExit();

	return EXIT_SUCCESS;
}
